import { KeyOf } from "./core";
export { IfKeyIsOptional, IfKeysOverlap, IfAllInUnionExtend, IfNoneInUnionExtend, IfSomeInUnionExtend, IfSomeInUnionDoNotExtend, };
/**
 * Conditionally check if property of an interface is optional
 */
declare type IfKeyIsOptional<T, K extends keyof T, TRUE = true, FALSE = false> = {
    [K2 in K]: T[K2];
} extends {
    [K2 in K]-?: T[K2];
} ? FALSE : TRUE;
/**
 * Conditionally check if any keys exist in both unions.
 */
declare type IfKeysOverlap<T extends KeyOf, U extends KeyOf, TRUE = true, FALSE = false> = Extract<T, U> | Extract<U, T> extends never ? FALSE : TRUE;
/** Conditional that tests if all types in a union extend a type */
declare type IfAllInUnionExtend<Union, Match, ResultIfAllMatch, ResultIfSomeDoNotMatch> = (Union extends Match ? never : {
    __fail_marker: any;
}) extends never ? ResultIfAllMatch : ResultIfSomeDoNotMatch;
/** Conditional that tests if all types in a union *do not* extend a type */
declare type IfNoneInUnionExtend<Union, Match, ResultIfNoneMatch, ResultIfSomeDoMatch> = (Union extends Match ? {
    __fail_marker: any;
} : never) extends never ? ResultIfNoneMatch : ResultIfSomeDoMatch;
/** Conditional that tests if at least one type in a union extends a type */
declare type IfSomeInUnionExtend<Union, Match, ResultIfSomeMatch, ResultIfNoneMatch> = IfNoneInUnionExtend<Union, Match, ResultIfNoneMatch, ResultIfSomeMatch>;
/** Conditional that tests if at least one type in a union *does not* extend a type */
declare type IfSomeInUnionDoNotExtend<Union, Match, ResultIfSomeDoNotMatch, ResultIfAllMatch> = IfAllInUnionExtend<Union, Match, ResultIfAllMatch, ResultIfSomeDoNotMatch>;
//# sourceMappingURL=conditionals.d.ts.map