"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function narrowLiterals(array) { return array; }
exports.narrowLiterals = narrowLiterals;
function typeOfExpression() { }
exports.typeOfExpression = typeOfExpression;
;
/**
 * Write type annotations inline within nested object or array literals.
 * Use this to enable better Intellisense and write self-documenting code.
 *
 * This is better than TS type assertions (e.g. `<MyInterface>{}`) because
 * type assertions allow type narrowing, which suppresses certain type errors.
 *
 * For example, in the following situation, Intellisense is useless without an explicit
 * type annotation enabled via this function:
 *
 *     const a: Dictionary<any> = {
 *         first: someValue,
 *         second: someOtherValue,
 *         windowShim: T<Partial<Window>>({ // We want code completion for properties of the window object
 *             // <- intellisense shows us setTimeout, document, alert, btoa, location, onblur, etc.
 *         })
 */
function T(value) { return value; }
exports.T = T;
/**
 * Short for "Type Inferred"
 * Write a type requirement inline within nested object or array literals, but allow inference to
 * narrow the type even further.
 * Use this to enable better Intellisense and write self-documenting code.
 *
 * This is better than TS type assertions (e.g. `<MyInterface>{}`) because
 * type assertions allow type narrowing, which suppresses certain type errors.
 *
 * *NOTE: usage requires an extra set of empty parentheses.  This is a generics trick.*
 *
 * For example, in the following situation, Intellisense is useless without an explicit
 * type annotation enabled via this function:
 *
 *     const a: Dictionary<any> = {
 *         first: someValue,
 *         second: someOtherValue,
 *         windowShim: TI<Partial<Window>>()({ // We want code completion for properties of the window object
 *             // <- intellisense shows us setTimeout, document, alert, btoa, location, onblur, etc.
 *         })
 */
function TI() {
    return T;
}
exports.TI = TI;
//# sourceMappingURL=core-values.js.map